
<!DOCTYPE html>

<html>
    <body>
        {% if filedata %}
        <h3>{{ filedata.filename }}</h3>
        {% endif %}
        <form method="POST" enctype="multipart/form-data">
            {{ form.hidden_tag() }}
            <p>
                {{ form.upload.label }}
                {{ form.upload }}
            </p>
            <p>
                <input type="submit" value="Submit">
            </p>
        </form>
    </body>
</html>
